import style from "./style.module.css";
import { useEffect, useState } from "react";
import { Close } from "@mui/icons-material";
import { IconButton } from "@mui/material";

enum Status {
  normal,
  addFile,
  receive,
}

interface Props {
  isSignIn: boolean;
  onNotSignIn: () => void;
}

export function HomeButton(props: Props) {
  let [status, setStatus] = useState(Status.normal);
  let [boxStyle, setBoxStyle] = useState([style.box]);
  let [isReceive, setIsReceive] = useState(false);
  let onNormal = () => {
    setStatus(Status.normal);
  };
  let onAddFile = () => {
    setStatus(Status.addFile);
  };
  let onReceive = () => {
    setStatus(Status.receive);
  };

  useEffect(() => {
    switch (status) {
      case Status.normal:
        setBoxStyle([style.box]);
        break;
      case Status.addFile:
        setBoxStyle([style.box, style.boxActivePlus]);
        break;
      case Status.receive:
        setBoxStyle([style.box, style.boxActiveReceive]);
        break;
    }
  }, [status]);

  return (
    <div className={boxStyle.join(" ")}>
      {!isReceive ? (
        <>
          <div className={style.plus}>➕</div>
          <div
            className={style.addFile}
            onMouseEnter={onAddFile}
            onMouseLeave={onNormal}
          >
            添加文件
          </div>
          <button
            className={style.receive}
            onMouseEnter={onReceive}
            onMouseLeave={onNormal}
            onClick={() => {
              if (props.isSignIn) {
                setIsReceive(true);
              } else {
                props.onNotSignIn();
              }
            }}
          >
            接收文件
          </button>
        </>
      ) : (
        <>
          <input
            placeholder={"输入6位传输口令获取文件"}
            style={{}}
            className={isReceive ? style.receiveInput : style.receiveInputNone}
          />
          <IconButton
            className={style.receiveInputClose}
            aria-label="delete"
            onClick={() => {
              setIsReceive(false);
            }}
          >
            <Close />
          </IconButton>
        </>
      )}
    </div>
  );
}
